-- ------------------------
-- TABELLEN und BEZIEHUNGEN
-- ------------------------

-- Definition der Tabelle Gemeinden

-- Anlegen der Tabelle:
CREATE TABLE Gemeinden (
  gkz   DECIMAL(8),                    -- Gemeindeschlssel
  name  VARCHAR(30)  NOT NULL,         -- Gemeindename
  einw  DECIMAL(7)   DEFAULT 0,        -- Einwohnerzahl
  CONSTRAINT pk_gem PRIMARY KEY (gkz)  -- Definition Primrschlssel
);

-- ndern der Tabelle:
ALTER TABLE Gemeinden ADD CONSTRAINT ck_gem_einw CHECK (einw>=0);

-- Lschen der Tabelle:
-- DROP TABLE Gemeinden;

-- Einfgen neuer Datenstze:
INSERT INTO Gemeinden (gkz,name,einw)
       VALUES (3241001,'Hannover',349149);
INSERT INTO Gemeinden (gkz,name) VALUES (3403000,'Oldenburg');
COMMIT;

-- ndern vorhandener Datenstze:
UPDATE Gemeinden SET einw = 157267 WHERE gkz=3403000;
UPDATE Gemeinden SET einw = einw +
                  (SELECT einw FROM Gemeinden WHERE gkz=3403000)
WHERE Name = 'Hannover';
COMMIT;

-- Lschen von Datenstzen und Rcknahme des Lschens:
DELETE FROM Gemeinden WHERE name = 'Hannover';
ROLLBACK;

-- Einfgen weiterer Datenstze:
INSERT INTO Gemeinden (gkz,name,einw) VALUES (3451002,'Bad Zwischenahn',27437);
INSERT INTO Gemeinden (gkz,name,einw) VALUES (3451007,'Westerstede',21941);
INSERT INTO Gemeinden (gkz,name,einw) VALUES (3461004,'Elsfleth',9196);
INSERT INTO Gemeinden (gkz,name,einw) VALUES (3404000,'Osnabrck',153699);
INSERT INTO Gemeinden (gkz,name,einw) VALUES (3405000,'Wilhelmshaven',77451);
COMMIT;

-- Definition eines Index

-- Index anlegen:
CREATE INDEX Gemeinden_name_ix ON Gemeinden(name);

-- Index wieder lschen:
DROP INDEX Gemeinden_name_ix;

-- Definition der Tabelle Kreise

CREATE TABLE Kreise (
  kkz        DECIMAL(5),                 -- Kreisschlssel
  name       VARCHAR(20) NOT NULL,       -- Name des Kreises
  qkm        DECIMAL(8,2),               -- Flche in qkm
  kreisstadt DECIMAL(8),                 -- GKZ der Kreisstadt
  CONSTRAINT pk_krs PRIMARY KEY (kkz),   -- Primrschlssel
  CONSTRAINT ck_krs_qkm CHECK (qkm > 0), -- Integrittsregel
  CONSTRAINT fk_krs_kst FOREIGN KEY (kreisstadt)
    REFERENCES Gemeinden(gkz)            -- Fremdschlssel
);

-- Einfgen der Datenstze in Kreise:
INSERT INTO Kreise VALUES (3403,'Oldenburg Stadt',102.97,3403000);
INSERT INTO Kreise VALUES (3451,'Ammerland',728.16,3451007);
INSERT INTO Kreise VALUES (3404,'Osnabrck Stadt',119.80,3404000);
INSERT INTO Kreise VALUES (3459,'Osnabrck Land',2121.56,3404000);
COMMIT;

-- Definition der Tabellen Hochschulen und HatStandortIn

CREATE TABLE Hochschulen (
  num    INTEGER,                -- Hochschulschlssel
  name   VARCHAR(20)  NOT NULL,  -- Hochschulname
  stud   DECIMAL(6),             -- Zahl der Studierenden
  CONSTRAINT pk_hs PRIMARY KEY (num)
);

CREATE TABLE HatStandortIn (
  hs        INTEGER,                               -- Hochschulschlssel
  gemeinde  DECIMAL(8),                            -- GKZ des Standorts
  CONSTRAINT pk_stdort PRIMARY KEY (hs,gemeinde),  -- komb. Primrschlssel
  CONSTRAINT fk_stdort_hs FOREIGN KEY (hs)
    REFERENCES Hochschulen(num) ON DELETE CASCADE, -- 1. Fremdschlssel
  CONSTRAINT fk_stdort_gem FOREIGN KEY (gemeinde)
    REFERENCES Gemeinden(gkz) ON DELETE CASCADE    -- 2. Fremdschlssel
);
  
-- Einfgen der Datenstze in Hochschulen:
INSERT INTO Hochschulen VALUES (1,'Uni Oldenburg',12000);
INSERT INTO Hochschulen VALUES (2,'Jade Hochschule',6700); 
INSERT INTO Hochschulen VALUES (3,'Uni Hannover',22000);
COMMIT;

-- Einfgen der Datenstze in HatStandortIn:
INSERT INTO HatStandortIn VALUES (3,3241001);
INSERT INTO HatStandortIn VALUES (1,3403000);
INSERT INTO HatStandortIn VALUES (2,3403000);
INSERT INTO HatStandortIn VALUES (2,3461004);
INSERT INTO HatStandortIn VALUES (2,3405000);
COMMIT;


-- --------
-- ANFRAGEN
-- --------

-- Projektion

-- Projektion:
SELECT name, einw FROM Gemeinden;
-- keine Projektion:
SELECT * FROM Gemeinden;

-- Selektion

-- Auswahl der Grostdte:
SELECT * FROM Gemeinden WHERE einw >= 100000;
-- Auswahl der Gemeinden, deren Name mit W beginnt oder
-- die weniger als 50000 Einwohner haben:
SELECT name, einw FROM Gemeinden WHERE name LIKE 'W%' OR einw < 50000;

-- Verbund

-- Die Kreisnamen mit den Namen der zugehrigen Kreisstdte:
SELECT Kreise.name, Gemeinden.name
FROM Kreise, Gemeinden
WHERE kreisstadt = gkz;

--- SQL-92-Variante:
SELECT Kreise.name, Gemeinden.name
FROM Kreise INNER JOIN Gemeinden
ON kreisstadt = gkz;

-- Outer Join (alte Oracle-Version):
SELECT Gemeinden.name, Kreise.name
FROM Gemeinden,Kreise
WHERE gkz = kreisstadt (+);

-- Outer Join gem SQL-92:
SELECT Gemeinden.name, Kreise.name
FROM Gemeinden LEFT JOIN Kreise
ON gkz = kreisstadt;

SELECT Gemeinden.name, Kreise.name
FROM Kreise RIGHT JOIN Gemeinden
ON gkz = kreisstadt;

-- Kartesisches Produkt:
SELECT Kreise.name, Gemeinden.name
FROM Kreise, Gemeinden;

-- Kartesisches Produkt gem SQL-92:
SELECT Kreise.name, Gemeinden.name
FROM Kreise CROSS JOIN Gemeinden;

-- IN-Bedingungen

SELECT name
FROM Gemeinden
WHERE gkz IN (SELECT kreisstadt FROM Kreise);

-- Sortierung und Entfernung von Duplikaten

-- aufsteigende Sortierung nach dem Namen:
SELECT gkz, name FROM Gemeinden ORDER BY name;
-- absteigende Sortierung nach der Einwohnerzahl:
SELECT gkz, name FROM Gemeinden ORDER BY einw DESC;

-- Aggregatsfunktionen

-- Projektion mit Duplikaten:
SELECT hs FROM HatStandortIn;
-- Projektion mit Entfernung von Duplikaten:
SELECT DISTINCT hs FROM HatStandortIn;

-- Anzahl der Gemeinden:
SELECT COUNT(*) FROM Gemeinden;
-- Durchschnittliche Einwohnerzahl der greren Gemeinden:
SELECT AVG(einw) FROM Gemeinden WHERE einw > 50000;

-- Gruppierung

-- Gruppierung ohne Bedingung:
SELECT gemeinde, COUNT(hs)
FROM HatStandortIn
GROUP BY gemeinde;

-- Gruppierung mit Bedingung:
SELECT gemeinde, COUNT(hs)
FROM HatStandortIn
GROUP BY gemeinde
HAVING COUNT(hs) > 1;

-- Definition von Sichten

-- Anlegen der Sicht:
CREATE VIEW Grossstdte AS
SELECT * FROM Gemeinden WHERE einw >= 100000;

-- Verwenden der Sicht:
SELECT name, einw FROM Grossstdte;

-- Lschen der Sicht:
DROP VIEW Grossstdte;
