-- ----------------------------
-- DIREKTE SPEICHERUNG in BLOBs
-- ----------------------------

-- ggf. Aufrumen:
DELETE FROM Rasterkarten;
DELETE FROM USER_SDO_GEOM_METADATA WHERE table_name = 'RASTERKARTEN';
DROP INDEX Rasterkarten_georef_ix;
ALTER TABLE Rasterkarten DROP(georef);
COMMIT;

-- Rasterkarte ber Java-Programm StoreRasterInBlob speichern.

-- Erweiterung der Rasterdaten-Tabelle:
ALTER TABLE Rasterkarten ADD georef SDO_GEOMETRY;

-- Metadaten einfgen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('Rasterkarten','georef', SDO_DIM_ARRAY(
  SDO_DIM_ELEMENT('Rechtswert',3200000,4000000,1),
  SDO_DIM_ELEMENT('Hochwert',5200000,6500000,1)
) , 31467);
COMMIT;

-- Index anlegen:
CREATE INDEX Rasterkarten_georef_ix
ON Rasterkarten(georef)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- Rasterkarte aktualisieren:
UPDATE Rasterkarten
SET georef = SDO_GEOMETRY(2003,31467,NULL,
               SDO_ELEM_INFO_ARRAY(1,1003,3),
               SDO_ORDINATE_ARRAY(3430000,5870000,3470000,5910000))
WHERE ID = 1;
COMMIT;

-- Rumliche Anfrage stellen:
SELECT name
FROM Rasterkarten
WHERE SDO_FILTER (georef, SDO_GEOMETRY(2003,31467,null,
                    SDO_ELEM_INFO_ARRAY(1,1003,3),
                    SDO_ORDINATE_ARRAY(337000,5700000,3600000,6000000))) = 'TRUE';

-- ber Java-Programm ReadRasterFromBlob BLOB einlesen und als Datei speichern.


-- -----------------
-- GeoRaster-OBJEKTE
-- -----------------

--- ggf. Aufrumen:
DROP TABLE Raster;
DROP TABLE GeoRasterkarten;
DELETE FROM USER_SDO_GEOM_METADATA WHERE table_name = 'GEORASTERKARTEN';
COMMIT;

--- Tabellen anlegen

-- Tabelle mit GeoRaster-Attribut anlegen:
CREATE TABLE GeoRasterkarten (
  id         DECIMAL(8),                  -- ID
  name       VARCHAR(32)  NOT NULL,       -- Name der Karte
  georaster  SDO_GEORASTER,               -- das Georaster
  CONSTRAINT pk_georst PRIMARY KEY (id)   -- Primrschlssel 
);

-- Trigger zur Datenintegritt anlegen:
CALL SDO_GEOR_UTL.createDMLTrigger('GeoRasterkarten', 'georaster');

-- Raster-Objekttabelle anlegen:
CREATE TABLE Raster OF SDO_RASTER (
  CONSTRAINT pk_rst PRIMARY KEY (rasterId, pyramidLevel, bandBlockNumber,
                                 rowBlockNumber, columnBlockNumber)  -- Primrschlsseldefinition
)
LOB(rasterblock) STORE AS (
  NOCACHE        -- keine Pufferung
  NOLOGGING      -- kein Logging
);

-- Leeres GeoRaster einfgen:
INSERT INTO GeoRasterkarten 
    VALUES( 1, 'OldenburgerLand.gif', SDO_GEOR.init('Raster', 1) );
COMMIT;

-- Laden ber SDO_GEOR.importFrom

-- vom DB-Administrator mssen Rechte vergeben werden:
CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand.gif', 'read' );
CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand.giw', 'read' );

-- Import durchfhren:
DECLARE
  geor SDO_GEORASTER;
BEGIN
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 1 FOR UPDATE;
  SDO_GEOR.importFrom(geor, 'blocking=false', 'GIF', 'FILE', 'C:\Temp\OldenburgerLand.gif',
                      'WORLDFILE', 'FILE', 'C:\Temp\OldenburgerLand.giw,31467');
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 1;
  COMMIT;
END;
/


-- Abfragen der Daten

-- Rastertyp, Name der Raster-Objekttabelle, Raster-ID bestimmen:
SELECT gr.georaster.rasterType, gr.geoRaster.rasterDataTable, gr.georaster.rasterId
FROM GeoRasterkarten gr;

-- Rumliche Ausdehnung des GeoRasters bestimmen (leer):
SELECT gr.georaster.spatialExtent FROM GeoRasterkarten gr;

-- Metadaten des Georasters bestimmen:
SELECT gr.georaster.metadata.getClobVal() FROM GeoRasterkarten gr;

-- Georasterbezug und Blocknummern der Kacheln in der Raster-Objekttabelle abfragen:
SELECT rasterid, rowblocknumber, columnblocknumber FROM Raster;

-- Rasterdatenraum der Kacheln bestimmen:
SELECT blockmbr FROM Raster;

-- Nachbereitung des Ladens

DECLARE
  geor SDO_GEORASTER;   -- das Georaster
  srs  SDO_GEOR_SRS;    -- ID des rumlichen Bezugsystems
  ext  SDO_GEOMETRY;    -- Datenraum
BEGIN
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 1 FOR UPDATE;
  geor.spatialextent := SDO_GEOR.generateSpatialExtent(geor);
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 1;
  COMMIT;
END;
/

-- Bestimmung der rumlichen Ausdehnung des GeoRasters:
SELECT gr.georaster.spatialExtent FROM GeoRasterkarten gr;

-- Indexierung

-- Metadaten einfgen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GeoRasterkarten','georaster.spatialextent', SDO_DIM_ARRAY(
  SDO_DIM_ELEMENT('Rechtswert',3200000,4000000,1),
  SDO_DIM_ELEMENT('Hochwert',5200000,6500000,1)
) , 31467);
COMMIT;

-- Index anlegen:
CREATE INDEX GeoRasterkarten_georst_ix
ON GeoRasterkarten(georaster.spatialextent)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- Rumliche Anfrage:
SELECT name
FROM GeoRasterkarten gr
WHERE SDO_FILTER (gr.georaster.spatialextent, SDO_GEOMETRY(2003,31467,null,
                    SDO_ELEM_INFO_ARRAY(1,1003,3),
                    SDO_ORDINATE_ARRAY(337000,5700000,3600000,6000000))) = 'TRUE';

-- Kachelung

-- Kachelung durchfhren:
DECLARE
  gr1 SDO_GEORASTER;   -- das Quell-GeoRaster
  gr3 SDO_GEORASTER;   -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten VALUES (2,'OldenburgerLand.gif',SDO_GEOR.INIT('Raster',2));
  SELECT georaster INTO gr1 FROM GeoRasterkarten WHERE id = 1;
  SELECT georaster INTO gr3 FROM GeoRasterkarten WHERE id = 2 FOR UPDATE;
  SDO_GEOR.changeFormatCopy(gr1, 'blocking=true blockSize=(512,256)',gr3);
  UPDATE GeoRasterkarten SET georaster = gr3 WHERE id = 2;
  DELETE FROM GeoRasterkarten WHERE id = 1;
  COMMIT;
END;
/

-- Kachelung berprfen:
SELECT rowblocknumber r,columnblocknumber c, blockmbr 
FROM Raster
ORDER BY rowblocknumber,columnblocknumber;

-- Validieren des Rasters
SELECT gr.id,
       SDO_GEOR.validateGeoRaster(gr.georaster) isvalid
FROM GeoRasterkarten gr ORDER BY id;

-- Anfragen und Analysen

-- Rasterzelle ber Pixelposition bestimmen:
SELECT SDO_GEOR.getCellValue(gr.georaster,0,450,300,0) FROM GeoRasterkarten gr;

-- Pixelposition fr Punkt bestimmen:
SELECT SDO_GEOR.getCellCoordinate(gr.georaster,0,
         SDO_GEOMETRY('POINT(3450000 5880000)',31467))
FROM GeoRasterkarten gr;

-- Rasterzelle ber Punkt bestimmen:
SELECT SDO_GEOR.getCellValue(gr.georaster,0,
         SDO_GEOMETRY('POINT(3450000 5880000)',31467),1)
FROM GeoRasterkarten gr;

-- Funktion zum Bestimmen eines Farbwertes aus einer Farbtabelle:
CREATE OR REPLACE FUNCTION colorValue (i DECIMAL, map SDO_GEOR_COLORMAP) RETURN VARCHAR
IS
BEGIN
  RETURN map.red(i) || ' ' || map.green(i) || ' ' || map.blue(i);
END;
/

-- Nutzung der Funktion:
SELECT colorValue(SDO_GEOR.getCellValue(gr.georaster,0,450,300,0),
                  SDO_GEOR.getColorMap(gr.georaster,1))
FROM GeoRasterkarten gr
WHERE id = 2;

-- Teilausschnitt bilden:
DECLARE
  gr2 SDO_GEORASTER;   -- das Quell-GeoRaster
  gr7 SDO_GEORASTER;   -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten VALUES (7,'Teilausschnitt',SDO_GEOR.INIT('Raster',7));
  SELECT georaster INTO gr2 FROM GeoRasterkarten WHERE id = 2;
  SELECT georaster INTO gr7 FROM GeoRasterkarten WHERE id = 7 FOR UPDATE;
  SDO_GEOR.subset(gr2, SDO_GEOMETRY(2003,31467,null,
                    SDO_ELEM_INFO_ARRAY(1,1003,3),
                    SDO_ORDINATE_ARRAY(344000,5880000,3460000,5900000)), '1','blocking=false',gr7);
  UPDATE GeoRasterkarten SET georaster = gr7 WHERE id = 7;
END;
/

-- Bildpyramide

-- Bildpyramide (ohne Kachelung) erzeugen:
DECLARE
  gr2 SDO_GEORASTER;   -- das Quell-GeoRaster
  gr4 SDO_GEORASTER;   -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten VALUES (4,'OldenburgerLand.png',SDO_GEOR.INIT('Raster',4));
  SELECT georaster INTO gr2 FROM GeoRasterkarten WHERE id = 2;
  SELECT georaster INTO gr4 FROM GeoRasterkarten WHERE id = 4 FOR UPDATE;
  SDO_GEOR.changeFormatCopy(gr2, 'blocking=false',gr4);       -- Kachelung aufheben
  SDO_GEOR.generatePyramid(gr4, 'rLevel=2, resampling=NN');   -- Pyramide anlegen
  UPDATE GeoRasterkarten SET georaster = gr4 WHERE id = 4;
  DELETE FROM GeoRasterkarten WHERE id = 2;
  COMMIT;
END;
/

-- Bildpyramide berprfen:
SELECT pyramidlevel AS pl, blockmbr 
FROM Raster
ORDER BY pyramidlevel;

-- Raster komprimieren

-- ber Java-Anwendung GeoRasterLoader OldenburgerLand.png ohne Farbtabelle laden

CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand.png', 'read' );
CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand.pnw', 'read' );

CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand-nonint.png', 'read' );
CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand-int.png', 'read' );

-- Import durchfhren:
DECLARE
  geor SDO_GEORASTER;
  srs  SDO_GEOR_SRS;
  ext  SDO_GEOMETRY;
BEGIN
  INSERT INTO GeoRasterkarten VALUES( 10, 'OldenburgerLand.png', SDO_GEOR.init('Raster', 10) );
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 10 FOR UPDATE;
  SDO_GEOR.importFrom(geor, 'blocking=false', 'PNG', 'FILE', 'C:\Temp\OldenburgerLand-nonint.png',
                      'WORLDFILE', 'FILE', 'C:\Temp\OldenburgerLand.pnw');
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 10;
  COMMIT;
END;
/

-- komprimieren 
DECLARE
  gr10 SDO_GEORASTER;   -- das Quell-GeoRaster
  gr11 SDO_GEORASTER;   -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten VALUES (11,'OldenburgerLand.png',SDO_GEOR.INIT('Raster',11));
  SELECT georaster INTO gr10 FROM GeoRasterkarten WHERE id = 10;
  SELECT georaster INTO gr11 FROM GeoRasterkarten WHERE id = 11 FOR UPDATE;
  SDO_GEOR.changeFormatCopy(gr10, 'compression=JPEG-F quality=75',gr11);
  UPDATE GeoRasterkarten SET georaster = gr11 WHERE id = 11;
  COMMIT;
END;
/

-- Exportieren ber SDO_GEOR.exportTo

-- vom DB-Administrator mssen Rechte vergeben werden:
CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand.tif', 'write' );
CALL dbms_java.grant_permission('PUBLIC','SYS:java.io.FilePermission',
  'C:\Temp\OldenburgerLand.tfw', 'write' );

-- Export durchfhren:
DECLARE
  geor SDO_GEORASTER;
BEGIN
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 4;
  SDO_GEOR.exportTo(geor, 'pLevel=0', 'TIFF', 'FILE', 'C:\Temp\OldenburgerLand',
                      'WORLDFILE', 'FILE', 'C:\Temp\OldenburgerLand');
END;
/

