-- --
-- 3D
-- --

-- Anlegen der Tabelle 

CREATE TABLE GeoDbLand3D (
  id     INTEGER,                          -- Identifikationsnummer
  name   VARCHAR(20),                      -- Objektname
  geo    SDO_GEOMETRY,                     -- Objektgeometrie
  CONSTRAINT pk_gdbland3D PRIMARY KEY(id)  -- Primrschlssel
);

-- Einfgen der 3D-Daten

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (1,'Flaggenmast',SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10,11,5),NULL,NULL));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (2,'Steg',SDO_GEOMETRY(3002,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,2,1),SDO_ORDINATE_ARRAY(4,4,1, 5,7,1.5, 8,8,1)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (3,'Baumreihe',SDO_GEOMETRY(3005,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1,3),
                      SDO_ORDINATE_ARRAY(10,19,5, 12,19,10, 14,19,7)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (5,'Haus',SDO_GEOMETRY(3003,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1003,3),SDO_ORDINATE_ARRAY(9,9,7, 12,10,7)));

INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (6,'Land',SDO_GEOMETRY(3007,NULL,NULL,SDO_ELEM_INFO_ARRAY(1,1003,1, 16,2003,1, 43,1003,1),
SDO_ORDINATE_ARRAY(1,1,1,19,1,1,19,20,1,1,20,1,1,1,1,4,5,1,3,10,1,7,14,1,13,15,1,17,12,1,16,8,1,13,5,1,9,3,1,4,5,1,7,7,1,12,7,1,13,11,1,10,13,1,7,10,1,7,7,1)));

COMMIT;

-- Metadaten bekannt geben

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEODBLAND3D','GEO', SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X',0,20,0.001),
                                           SDO_DIM_ELEMENT('Y',0,20,0.001),
                                           SDO_DIM_ELEMENT('Z',0,10,0.001)) ,
        NULL);
COMMIT;

-- Geometrieprfung

SELECT id FROM GeoDbLand3D WHERE SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,0.1) <> 'TRUE';

-- Geometrische Funktionen und Methoden

--- Abfrage Dimension:
SELECT name, g.geo.get_dims() FROM GeoDbLand3D g;

-- minimal umgebendes Rechteck:
SELECT SDO_GEOM.SDO_MBR(geo) FROM GeoDbLand3D WHERE name = 'Steg';

-- Lnge (auf 2D projiziert!):
SELECT name, SDO_GEOM.SDO_LENGTH(geo,0.1) FROM GeoDbLand3D WHERE name = 'Steg';

-- R-Baum als Index anlegen

CREATE INDEX GeoDbLand3D_geo_ix
ON GeoDbLand3D(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX
PARAMETERS ('SDO_INDX_DIMS=3');

-- Rechteckanfrage
SELECT name
FROM GeoDbLand3D
WHERE SDO_FILTER (geo, SDO_GEOMETRY(3003,NULL,NULL,
                         SDO_ELEM_INFO_ARRAY(1,1003,3),
                         SDO_ORDINATE_ARRAY(7,7,1.5, 15,15,5))) = 'TRUE';